/******************************************************************************
 *	Some utility functions
 *****************************************************************************/

// Reserved browser information
$.browser = {};
$.browser.mozilla = /mozilla/.test(navigator.userAgent.toLowerCase()) && !/webkit/.test(navigator.userAgent.toLowerCase());
$.browser.webkit = /webkit/.test(navigator.userAgent.toLowerCase());
$.browser.opera = /opera/.test(navigator.userAgent.toLowerCase());
$.browser.msie = /msie/.test(navigator.userAgent.toLowerCase());

// Schedule to deselect everything on page
var deselectEverything = function(delay) {
	if (delay < 0) {
		document.getSelection().removeAllRanges();
	} else {
		setTimeout(function(){
			document.getSelection().removeAllRanges();
		}, delay);
	}
}

// Get postfix for given state
var getPostfixForState = function(state) {
	if (state == 0) {
		return "__Normal";
	} else if (state == 16) {
		return "__Hovering";
	} else if (state == 32) {
		return "__Selected";
	} else if (state == 64) {
		return "__Disabled";
	} else if (state == 48) {
		return "__Selected_Hovering";
	} else if (state == 96) {
		return "__Selected_Disabled";
	}
	return "";
}

// Create canvas with given width and height, enlarge and offset for shadow if needed
var createCanvas = function(w, h, params) {
	var width = w;
	var height = h;
	var hasShadow = (params && params.shadow);
	if (hasShadow) {
		var blur2 = params.shadowBlur * 2;
		var dx = Math.max(0, Math.abs(params.shadowOffsetX) - Math.abs(params.shadowBlur));
		var dy = Math.max(0, Math.abs(params.shadowOffsetY) - Math.abs(params.shadowBlur));
		width += blur2 + dx;
		height += blur2 + dy;
	}
	var canvasObj;
	var canvas;
	if (typeof(G_vmlCanvasManager) == "undefined") {
		canvasObj = $("<canvas width=\"" + width + "\" height=\"" + height + "\"></canvas>");
		canvas = canvasObj.get(0);
	} else {
		canvas = document.createElement("canvas");
		canvas.setAttribute("width", width);
		canvas.setAttribute("height", height);
		G_vmlCanvasManager.initElement(canvas);
		canvasObj = $(canvas);
	}
	if (hasShadow) {
		var tx = params.shadowOffsetX - params.shadowBlur;
		var ty = params.shadowOffsetY - params.shadowBlur;
		canvasObj.css({
			"position" : "relative",
			"left" : tx + "px",
			"top" : ty + "px"
		});
	}
	return canvas;
};

// Get the canvas' painting context, setup shadow when needed
var getCanvasContext = function(canvas, params) {
	var ctx = canvas.getContext('2d');
	if (params && params.shadow) {
		ctx.shadowColor = params.shadowColor;
		ctx.shadowOffsetX = params.shadowOffsetX;
		ctx.shadowOffsetY = params.shadowOffsetY;
		ctx.shadowBlur = params.shadowBlur;
		ctx.translate(params.shadowBlur - params.shadowOffsetX, params.shadowBlur - params.shadowOffsetY);
	}
	return ctx;
};

var setLineStyle = function(ctx, color, lineWidth, lineStyle) {
	ctx.strokeStyle = color;
	ctx.lineWidth = lineWidth;
	ctx.lineCap = "round";
	var dash = 3 * lineWidth;
	if (lineStyle == 'dashed') {
		ctx.setLineDash([dash, dash]);
	} else if (lineStyle == 'dotted') {
		ctx.setLineDash([1, dash]);
	}
};

// Draw swing line on canvas
var drawSwingLine = function(ctx, p1, p2, interval, amplitude, appendMode, doStroke) {
	if (!appendMode) {
		ctx.moveTo(p1.x, p1.y);
	} else {
		ctx.lineTo(p1.x, p1.y);
	}
	var l = Math.round(Math.sqrt(Math.pow(p1.x - p2.x, 2) + Math.pow(p1.y - p2.y, 2)));
	if (interval > 0 && amplitude > 0 && l > 0) {
		var r = Math.floor(amplitude / 2);
		var dx = (p2.x - p1.x) / l;
		var dy = (p2.y - p1.y) / l;
		var x = 0;
		var y = 0;
		var n = Math.floor(l / interval);
		var xa = 0;
		var ya = 1;
		var f = new Array();
		f[0] = 0;
		f[1] = 1;
		for(var i = 2; i < n; i ++ ) {
			f[i] = f[i - 1] + f[i - 2];
		}
		for(var i = 1; i < n; i ++) {
			var xa = i * f[i - 1] % f[i] % amplitude - r;
			var yatmp = f[i - 1]  % amplitude - r;
			if(Math.abs(yatmp - ya) > r) {
				ya = 1;
			} else {
				ya = yatmp;
			}
			var j = i * interval;
			x = p1.x + Math.round(dx * j) + xa;
			y = p1.y + Math.round(dy * j) + ya;
			ctx.lineTo(x, y);
		}
	}
	ctx.lineTo(p2.x, p2.y);
	if (doStroke) {
		ctx.stroke();
	}
};

// Draw swing border on canvas
var drawSwingBorder = function(ctx, x, y, width, height,
    		borderT, borderR, borderB, borderL, radiusTL, radiusTR, radiusBR, radiusBL,
    		interval, amplitude) {
	var halfBorderWidthTop = borderT / 2;
	var halfBorderWidthRight = borderR / 2;
	var halfBorderWidthBottom = borderB / 2;
	var halfBorderWidthLeft = borderL / 2;

	var maxCornerRadius = Math.min(width - halfBorderWidthLeft - halfBorderWidthRight, height - halfBorderWidthTop - halfBorderWidthBottom) / 2;

	var radiusTopLeft = Math.min(radiusTL, maxCornerRadius);
	var radiusTopRight = Math.min(radiusTR, maxCornerRadius);
	var radiusBottomRight = Math.min(radiusBR, maxCornerRadius);
	var radiusBottomLeft = Math.min(radiusBL, maxCornerRadius);

	var tlOffsetX = halfBorderWidthLeft + radiusTopLeft;
	var tlOffsetY = halfBorderWidthTop + radiusTopLeft;
	var trOffsetX = halfBorderWidthLeft + radiusTopRight;
	var trOffsetY = halfBorderWidthTop + radiusTopRight;
	var brOffsetX = halfBorderWidthRight + radiusBottomRight;
	var brOffsetY = halfBorderWidthBottom + radiusBottomRight;
	var blOffsetX = halfBorderWidthLeft + radiusBottomLeft;
	var blOffsetY = halfBorderWidthBottom + radiusBottomLeft;

	// top
	var curX = x + tlOffsetX;
	var curY = y + halfBorderWidthTop;
	var nextX = x + width - trOffsetX - 1;
	if (borderT > 0) {
		ctx.lineWidth = borderT;
		ctx.beginPath();
		drawSwingLine(ctx, {x : curX, y : curY}, {x : nextX, y : curY}, interval, amplitude, false, true);
	}
	curX = nextX;

	// top right
	if (radiusTopRight > 0 && (borderT > 0 || borderR > 0)) {
		ctx.arc(curX, curY + radiusTopRight, radiusTopRight, 1.5 * Math.PI, 2 * Math.PI, false);
		ctx.stroke();
	}

	// right
	curX = x + width - halfBorderWidthRight - 1;
	curY = y + trOffsetY;
	nextY = y + height - brOffsetY - 1;
	if (borderR > 0) {
		ctx.lineWidth = borderR;
		ctx.beginPath();
		drawSwingLine(ctx, {x : curX, y : curY}, {x : curX, y : nextY}, interval, amplitude, false, true);
	}
	curY = nextY;

	// bottom right
	if (radiusBottomRight > 0 && (borderB > 0 || borderR > 0)) {
		ctx.arc(curX - radiusBottomRight, curY, radiusBottomRight, 0, 0.5 * Math.PI, false);
		ctx.stroke();
	}

	// bottom
	curX = x + width - 1 - brOffsetX;
	curY = y + height - halfBorderWidthBottom - 1;
	nextX = x + blOffsetX;
	if (borderB > 0) {
		ctx.lineWidth = borderB;
		ctx.beginPath();
		drawSwingLine(ctx, {x : curX, y : curY}, {x : nextX, y : curY}, interval, amplitude, false, true);
	}
	curX = nextX;

	// bottom left
	if (radiusBottomLeft > 0 && (borderB > 0 || borderL > 0)) {
		ctx.arc(curX, curY - radiusBottomLeft, radiusBottomLeft, 0.5 * Math.PI, Math.PI, false);
		ctx.stroke();
	}

	// left
	curX = x + halfBorderWidthLeft;
	curY = y + height - 1 - blOffsetY;
	nextY = y + tlOffsetY;
	if (borderL > 0) {
		ctx.lineWidth = borderL;
		ctx.beginPath();
		drawSwingLine(ctx, {x : curX, y : curY}, {x : curX, y : nextY}, interval, amplitude, false, true);
	}
	curY = nextY;

	// top left
	if (radiusTopLeft > 0 && (borderT > 0 || borderL > 0)) {
		ctx.arc(curX + radiusTopLeft, curY, radiusTopLeft, Math.PI, 1.5 * Math.PI, false);
		ctx.stroke();
	}
};

// Fill and stroke the path on canvas, with given parameters
var fillAndStrokeOnCanvas = function(ctx, width, height, params, nofill, noborder) {
	// background fill
	var hasFilled = false;
	if (!nofill) {
		if (params["bg_" + params.state].colors) {
			var bg = params["bg_" + params.state];
			var bkRegion = bg.bkRegion;
			var startX = bg.startX * width / bkRegion.width;
			var startY = bg.startY * height / bkRegion.height;
			var endX = bg.endX * width / bkRegion.width;
			var endY = bg.endY * height / bkRegion.height;
			var grd = ctx.createLinearGradient(startX, startY, endX, endY);
			var fractions = bg.fractions;
			var colors = bg.colors;
			for (var i = 0; i < fractions.length; i ++) {
				grd.addColorStop(fractions[i], colors[i]);
			}
			ctx.fillStyle = grd;
		} else {
			ctx.fillStyle = params["bg_" + params.state];
		}
		hasFilled = params["bgo_" + params.state] != 0;
		ctx.save();
		if (params.bgo !== undefined) {
			ctx.globalAlpha = params.bgo;
			hasFilled = ctx.globalAlpha != 0.0;
		}
		ctx.fill();
		ctx.restore();
	}
	// draw border
	if (!noborder && params["bd_paint_" + params.state]) {
		ctx.save();
		var fillStyle = params["bg_" + params.state];
		if(hasFilled) {
			ctx.shadowColor = "transparent";
		}
		ctx.lineWidth = params["bdw_" + params.state];
		ctx.strokeStyle = params["bdc_" + params.state];
		ctx.stroke();
		ctx.restore();
	}
};

// Draw ellipse on canvas
var drawEllipse = function(ctx, x, y, w, h, params, doStroke) {
	var k = 0.5522848;
	var ox = (w / 2) * k;
	var oy = (h / 2) * k;
	var xe = x + w;
	var ye = y + h;
	var xm = x + w / 2;
	var ym = y + h / 2;
	ctx.beginPath();
	ctx.moveTo(x, ym);
	ctx.bezierCurveTo(x, ym - oy, xm - ox, y, xm, y);
	ctx.bezierCurveTo(xm + ox, y, xe, ym - oy, xe, ym);
	ctx.bezierCurveTo(xe, ym + oy, xm + ox, ye, xm, ye);
	ctx.bezierCurveTo(xm - ox, ye, x, ym + oy, x, ym);
	ctx.closePath();
	if (doStroke) {
		ctx.lineWidth = params["bdw_" + params.state];
		ctx.strokeStyle = params["bdc_" + params.state];
		ctx.stroke();
	}
}

// Get the size of given text
var getTextSize = function(text, fontName, fontSize, bold) {
	var empty = false;
	if (!text || text == "") {
		empty = true;
		text = "H";
	}
	var label = $("<span style=\"position:absolute; z-index:-1\">" + text + "</span>").appendTo("body");
	if (fontName) {
		label.css("font-family", fontName);
	}
	if (fontSize) {
		label.css("font-size", fontSize);
	}
	if (bold) {
		label.css("font-weight", "bold");
	}
	var obj = label.get(0);
	var w = empty ? 0 : obj.clientWidth;
	var h = obj.clientHeight;
	label.remove();
	return { "width" : w, "height" : h };
}

// Get the scroll bar size
var getScrollBarSize = function() {
	var inner = $("<p style=\"width:100%;height:200px;\"></p>");
	var outer = $("<div style=\"position:absolute;top:0px;left:0px;visibility:hidden;width:200px;height:150px;overflow:hidden\"></div>").append(inner).appendTo("body");
	var w1 = inner.get(0).offsetWidth;
	outer.css("overflow", "scroll");
	var w2 = inner.get(0).offsetWidth;
	if (w1 == w2) {
		w2 = outer.get(0).clientWidth;
	}
	outer.remove();
	return (w1 - w2);
}

// Quickly rotate given rectangle
var rotateRectangle = function(rc) {
	var tmp = rc.x
	rc.x = rc.y;
	rc.y = tmp;
	tmp = rc.width;
	rc.width = rc.height;
	rc.height = tmp;
};

// Change background (color or image)
var changeBackground = function(state, targetId, bgObj, params, bg, bgWidth, bgHeight) {
	if (!bg) {
		bg = params[targetId + "bg_" + state];
	}
	if (bg.indexOf("#") == 0 || bg == "transparent" || bg.indexOf("rgba(") == 0) {
		bgObj.css("background-color", bg);
	} else {
		var imgWidth = bgWidth ? bgWidth : bgObj.innerWidth();
		var imgHeight = bgHeight ? bgHeight : bgObj.innerHeight();
		var img = $("<img class=\"linearBg\" src=\"images/" + bg + "\" width=\"" + imgWidth + "\" height=\"" + imgHeight + "\" align=\"top\" />");
		var radiusTopLeft = params[targetId + "bdr_tl_" + state];
		var radiusTopRight = params[targetId + "bdr_tr_" + state];
		var radiusBottomRight = params[targetId + "bdr_br_" + state];
		var radiusBottomLeft = params[targetId + "bdr_bl_" + state];
		img.css({
			"-moz-border-radius-topleft" : radiusTopLeft + "px",
			"-webkit-border-top-left-radius" : radiusTopLeft + "px",
			"border-top-left-radius" : radiusTopLeft + "px",
			"-moz-border-radius-topright" : radiusTopRight + "px",
			"-webkit-border-top-right-radius" : radiusTopRight + "px",
			"border-top-right-radius" : radiusTopRight + "px",
			"-moz-border-radius-bottomleft" : radiusBottomLeft + "px",
			"-webkit-border-bottom-left-radius" : radiusBottomLeft + "px",
			"border-bottom-left-radius" : radiusBottomLeft + "px",
			"-moz-border-radius-bottomright" : radiusBottomRight + "px",
			"-webkit-border-bottom-right-radius" : radiusBottomRight + "px",
			"border-bottom-right-radius" : radiusBottomRight + "px"
		});
		bgObj.children(".linearBg").remove();
		bgObj.prepend(img);
	}
}

// Change borders
var changeBorders = function(state, targetId, bdObj, params, updateRadius) {
	var borderType = params[targetId + "bdt_" + state];
	var borderColor = params[targetId + "bdc_" + state];
	var borderWidthTop = params[targetId + "bd_t_" + state];
	var borderWidthRight = params[targetId + "bd_r_" + state];
	var borderWidthBottom = params[targetId + "bd_b_" + state];
	var borderWidthLeft = params[targetId + "bd_l_" + state];
	var radiusTopLeft = params[targetId + "bdr_tl_" + state];
	var radiusTopRight = params[targetId + "bdr_tr_" + state];
	var radiusBottomRight = params[targetId + "bdr_br_" + state];
	var radiusBottomLeft = params[targetId + "bdr_bl_" + state];
	var swingInterval = params[targetId + "bdsw_int_" + state];
	var swingAmplitude = params[targetId + "bdsw_amp_" + state];
	if (borderType == "swing") {
		var bdWidth = bdObj.outerWidth();
		var bdHeight = bdObj.outerHeight();
		var offset = Math.floor(swingAmplitude / 2);
		var canvas = createCanvas(bdWidth, bdHeight);
		canvas.style.position = "absolute";
		canvas.className = "swingBd";
		bdObj.children(".swingBd").remove();
		bdObj.prepend(canvas);
		var ctx = canvas.getContext('2d');
		ctx.strokeStyle = borderColor;
		drawSwingBorder(ctx, offset, offset, bdWidth - 2 * offset, bdHeight - 2 * offset, borderWidthTop, borderWidthRight, borderWidthBottom, borderWidthLeft, radiusTopLeft, radiusTopRight, radiusBottomRight, radiusBottomLeft, swingInterval, swingAmplitude);
	} else if (borderType != "none") {
		bdObj.css({
			"border-style" : borderType,
			"border-color" : borderColor,
			"border-top-width" : borderWidthTop + "px",
			"border-left-width" : borderWidthLeft + "px",
			"border-bottom-width" : borderWidthBottom + "px",
			"border-right-width" : borderWidthRight + "px"
		});
		if (updateRadius) {
			bdObj.css({
				"-moz-border-radius-topleft" : radiusTopLeft + "px",
				"-webkit-border-top-left-radius" : radiusTopLeft + "px",
				"border-top-left-radius" : radiusTopLeft + "px",
				"-moz-border-radius-topright" : radiusTopRight + "px",
				"-webkit-border-top-right-radius" : radiusTopRight + "px",
				"border-top-right-radius" : radiusTopRight + "px",
				"-moz-border-radius-bottomleft" : radiusBottomLeft + "px",
				"-webkit-border-bottom-left-radius" : radiusBottomLeft + "px",
				"border-bottom-left-radius" : radiusBottomLeft + "px",
				"-moz-border-radius-bottomright" : radiusBottomRight + "px",
				"-webkit-border-bottom-right-radius" : radiusBottomRight + "px",
				"border-bottom-right-radius" : radiusBottomRight + "px"
			});
		}
	}
};

// Generate background and borders in given objects, with given parameters
var generateBackgroundAndBorders = function(state, targetId, bgObj, bdObj, params) {
	var bgWidth = bgObj.outerWidth();
	var bgHeight = bgObj.outerHeight();
	var bdWidth = bdObj.outerWidth();
	var bdHeight = bdObj.outerHeight();
	var bg = (targetId != "" || !params.bg) ? params[targetId + "bg_" + state] : params.bg;
	var opacity = params[targetId + "bgo_" + state];
	var shadow = params[targetId + "shd_" + state];
	var borderPaint = params[targetId + "bd_paint_" + state];
	var borderType = params[targetId + "bdt_" + state];
	var borderColor = params[targetId + "bdc_" + state];
	var borderWidthTop = params[targetId + "bd_t_" + state];
	var borderWidthRight = params[targetId + "bd_r_" + state];
	var borderWidthBottom = params[targetId + "bd_b_" + state];
	var borderWidthLeft = params[targetId + "bd_l_" + state];
	var radiusTopLeft = params[targetId + "bdr_tl_" + state];
	var radiusTopRight = params[targetId + "bdr_tr_" + state];
	var radiusBottomRight = params[targetId + "bdr_br_" + state];
	var radiusBottomLeft = params[targetId + "bdr_bl_" + state];
	var swingInterval = params[targetId + "bdsw_int_" + state];
	var swingAmplitude = params[targetId + "bdsw_amp_" + state];

	// background
	bgObj.children(".linearBg").remove();
	if (bg.indexOf("#") == 0 || bg == "transparent" || bg.indexOf("rgba(") == 0) {
		bgObj.css("background-color", bg);
	} else {
		var imgWidth = bgObj == bdObj ? bgWidth - borderWidthLeft - borderWidthRight : bgWidth;
		var imgHeight = bgObj == bdObj ? bgHeight - borderWidthTop - borderWidthBottom : bgHeight;
		bgObj.prepend($("<img class=\"linearBg\" src=\"images/" + bg + "\" width=\"" + imgWidth + "\" height=\"" + imgHeight + "\" align=\"top\" />"));
	}
	if (opacity != undefined) {
		bgObj.fadeTo(0, opacity / 255);
	}

	// shadow
	if (shadow) {
		bgObj.css({
			"box-shadow" : shadow,
			"-moz-box-shadow" : shadow,
			"-webkit-box-shadow" : shadow
		});
	}

	// border
	var roundedCss = {
		"-moz-border-radius-topleft" : radiusTopLeft + "px",
		"-webkit-border-top-left-radius" : radiusTopLeft + "px",
		"border-top-left-radius" : radiusTopLeft + "px",
		"-moz-border-radius-topright" : radiusTopRight + "px",
		"-webkit-border-top-right-radius" : radiusTopRight + "px",
		"border-top-right-radius" : radiusTopRight + "px",
		"-moz-border-radius-bottomleft" : radiusBottomLeft + "px",
		"-webkit-border-bottom-left-radius" : radiusBottomLeft + "px",
		"border-bottom-left-radius" : radiusBottomLeft + "px",
		"-moz-border-radius-bottomright" : radiusBottomRight + "px",
		"-webkit-border-bottom-right-radius" : radiusBottomRight + "px",
		"border-bottom-right-radius" : radiusBottomRight + "px"
	};
	if (borderPaint) {
		if (borderType == "swing") {
			var offset = Math.floor(swingAmplitude / 2);
			var canvas = createCanvas(bdWidth, bdHeight);
			canvas.style.position = "absolute";
			canvas.className = "swingBd";
			bdObj.children(".swingBd").remove();
			var firstChild = bdObj.children(":first");
			if (firstChild.length > 0 && firstChild.hasClass("labelBg")) {
				firstChild.after(canvas);
			} else {
				bdObj.prepend(canvas);
			}
			var ctx = canvas.getContext('2d');
			ctx.strokeStyle = borderColor;
			drawSwingBorder(ctx, offset, offset, bdWidth - 2 * offset, bdHeight - 2 * offset, borderWidthTop, borderWidthRight, borderWidthBottom, borderWidthLeft, radiusTopLeft, radiusTopRight, radiusBottomRight, radiusBottomLeft, swingInterval, swingAmplitude);
		} else if (borderType != "none") {
			var sizeCss = {
				"width" : (bdWidth - borderWidthLeft - borderWidthRight) + "px",
				"height" : (bdHeight - borderWidthTop - borderWidthBottom) + "px"
			};
			var borderCss = {
				"border-style" : borderType,
				"border-color" : borderColor,
				"border-top-width" : borderWidthTop + "px",
				"border-left-width" : borderWidthLeft + "px",
				"border-bottom-width" : borderWidthBottom + "px",
				"border-right-width" : borderWidthRight + "px"
			};
			bdObj.css(sizeCss).css(borderCss).css(roundedCss);
		}
	}
	bgObj.css(roundedCss).find("img").css(roundedCss);
};

// Apply border radius on given objects, with given parameters
var applyBorderRadius = function(state, targetId, bdObj, params) {
	var radiusTopLeft = params[targetId + "bdr_tl_" + state];
	var radiusTopRight = params[targetId + "bdr_tr_" + state];
	var radiusBottomRight = params[targetId + "bdr_br_" + state];
	var radiusBottomLeft = params[targetId + "bdr_bl_" + state];
	bdObj.css({
		"-moz-border-radius-topleft" : radiusTopLeft + "px",
		"-webkit-border-top-left-radius" : radiusTopLeft + "px",
		"border-top-left-radius" : radiusTopLeft + "px",
		"-moz-border-radius-topright" : radiusTopRight + "px",
		"-webkit-border-top-right-radius" : radiusTopRight + "px",
		"border-top-right-radius" : radiusTopRight + "px",
		"-moz-border-radius-bottomleft" : radiusBottomLeft + "px",
		"-webkit-border-bottom-left-radius" : radiusBottomLeft + "px",
		"border-bottom-left-radius" : radiusBottomLeft + "px",
		"-moz-border-radius-bottomright" : radiusBottomRight + "px",
		"-webkit-border-bottom-right-radius" : radiusBottomRight + "px",
		"border-bottom-right-radius" : radiusBottomRight + "px"
	});
};

// Apply font parameters on given object
var applyFontParameters = function(state, targetId, textObj, params) {
	var fontName = params[targetId + "f_n_" + state];
	var fontSize = params[targetId + "f_s_" + state];
	var textColor = params[targetId + "f_c_" + state];
	var bold = params[targetId + "f_b_" + state];
	var italic = params[targetId + "f_i_" + state];
	var underline = params[targetId + "f_u_" + state];
	textObj.css({
		"font-family" : fontName,
		"font-size" : fontSize,
		"color" : textColor,
		"font-weight" : bold ? "bold" : "normal",
		"font-style" : italic ? "italic" : "normal",
		"text-decoration" : underline ? "underline" : "none"
	});
};

// Create a label object that contains the given icon and text
var createLabelObject = function(icon, placement, gap, text, fromParam) {
	var label = $("<span class=\"label\" style=\"white-space:nowrap;\"></span>");
	if (icon) {
		var imgPathPrefix = fromParam ? "\"images/" : "\"images/dock/";
		var img = "<img class=\"icon\" src=" + imgPathPrefix + icon + "\" align=\"absmiddle\" />";
		var hGap = "<span style=\"display:inline-block; width:" + gap + "px; height:" + gap + "px\"/>";
		var vGap = "<div style=\"font-size:1px; width:" + gap + "px; height:" + gap + "px\"/>";
		var content = (placement % 2 == 0 ? img : text) + (placement > 1 ? vGap : hGap) + (placement % 2 == 0 ? text : img);
		label.html(content);
	} else {
		label.html(text);
	}
	return label;
};

// Initialize the references of given element
var initReferences = function(elem) {
	var params = elem._params;
	if (params.tooltip) {
		elem._obj.attr("title", params.tooltip);
	}
	var original = elem._obj.first();
	$("[ref=\"" + elem._id + "\"]").each(function() {
		var me = $(this);
		if (!me.attr("title") && params.tooltip) {
			me.attr("title", params.tooltip);
		}
		getForeUIElementById(me.attr("id"));
		me.empty().append(original.clone(false).children());
		me.find(".elem").each(function() {
			var emb = $(this);
			var orginId = emb.attr("id");
			emb.attr("id", me.attr("id") + "__" + orginId)
				.attr("ref", orginId);
		});
		if (elem.setupBehavior) {
			elem.setupBehavior(me);
		}
		elem.updateUI(me);
	});
};

// Update the element and its references
var updateElementAndReferences = function(elem) {
	elem.updateUI(elem._obj);
	$("[ref=\"" + elem._id + "\"]").each(function() {
		elem.updateUI($(this));
	});
};

// Check if the given DOM object is a page
var isPage = function(obj) {
	return obj != null && obj.id != null && obj.id.indexOf("Page_") == 0;
};

// Return the owner page of the given DOM object
var getPageByObject = function(obj) {
	while (obj = obj.parentNode) {
		if (isPage(obj)) {
			return obj;
		}
	}
	return null;
};

// Check if DOM object A can cover DOM object B
var canACoverB = function(objA, objB) {
	if (objA == null) {
		return false;
	}
	if (objB == null) {
		return true;
	}
	this.getActualZ = function(obj) {
		try {
			var z = obj.style.zIndex == "auto" ? 0 : obj.style.zIndex;
		} catch (err) {
			var z = 0;
		}
		if (typeof(z) != "number") {
			z = parseInt(z, 10);
		}
		return z;
	};
	this.getChildIndex = function(parentNode, node) {
		for (var i = 0; i < node.parentNode.childNodes.length; i ++){
			if(node.parentNode.childNodes[i] == node) {
				return i;
			}
		}
	};
	var hostA = isPage(objA.parentNode) ? objA : objA.parentNode;
	var hostB = isPage(objB.parentNode) ? objB : objB.parentNode;
	var nextA = isPage(hostA.parentNode) ? hostA : hostA.parentNode;
	var nextB = isPage(hostB.parentNode) ? hostB : hostB.parentNode;
	if (nextA != nextB && (nextA != hostA || nextB != hostB)) {
		return canACoverB(nextA, nextB);
	} else {
		var aIndex = this.getActualZ(hostA);
		var bIndex = this.getActualZ(hostB);
		if (aIndex == bIndex && hostA.parentNode == hostB.parentNode) {
			aIndex = this.getChildIndex(hostA.parentNode, hostA);
			bIndex = this.getChildIndex(hostB.parentNode, hostB);
		}
		return (aIndex >= bIndex);
	}
};

// Check if the given rectangle is located within another rectangle
var rectInRect = function(l, t, w, h, bL, bT, bW, bH) {
	this.ptInRect = function(x, y, l, t, w, h) {
		return (x >= l) && (x < (l + w)) && (y >= t) && (y < (t + h));
	};
	var ltIn = this.ptInRect(l, t, bL, bT, bW, bH);
	var lbIn = this.ptInRect(l, t + h - 1, bL, bT, bW, bH);
	var rtIn = this.ptInRect(l + w - 1, t, bL, bT, bW, bH);
	var rbIn = this.ptInRect(l + w - 1, t + h - 1, bL, bT, bW, bH);
	return ltIn && lbIn && rtIn && rbIn;
};

// Return the DOM objects under the given parent, located in given bounds and can cover the base object
var objectsInRect = function(parent, x, y, w, h, base) {
	if (!base) {
		base = parent;
	}
	var objs = [];
	$(".elem", parent).each(function() {
		if (canACoverB(this, base)) {
			var bounds = getBounds(this);
			if (rectInRect(bounds.x, bounds.y, bounds.w, bounds.h, x, y, w, h)) {
				objs.push(this);
			}
		}
	});
	return objs;
};

// Initialize the behavior for page or element specified by the given id
var initBehaviorForId = function(id) {
	var behavior = window.foreui.behaviors[id];
	if (behavior != undefined) {
		for (var i = 0; i < behavior.length; i ++) {
			if (behavior[i].executed == undefined && canExecuteBehavior(behavior[i])) {
				behavior[i].instance.callMe();
				behavior[i].executed = true;
			}
		}
	}
};

// Check if the given behavior can be executed
var canExecuteBehavior = function(behavior) {
	if (behavior.instance == undefined) {
		behavior.instance = new behavior();
	}
	var selector = behavior.instance.selector;
	var count = selector.size();
	if (count > 0) {
		for (var i = 0; i < count; i ++) {
			var id = selector.eq(i).attr("id");
			if (id.indexOf("Page_") == 0) {
				return true;
			}
			if (id.indexOf("Reference_") != 0 && !window.foreui.elements[id]) {
				return false;
			}
		}
		return true;
	}
	return false;
};

// Initialize the default behavior for given element
var initDefaultBehaviorForElement = function(element) {
	if (element) {
		var events = [
			"mouseenter",
			"mouseleave",
			"mousemove",
			"mouseover",
			"mouseout",
			"mousedown",
			"mouseup",
			"contextmenu",
			"click",
			"Selection_Changed",
			"Value_Changed"
		].join(" ");
		element._obj.bind(events, function(event) {
			if (element && (element._params.state & 64) > 0) {
				event.stopImmediatePropagation();
			}
		}).mousedown(function(event) {
			if ($(event.target).parentsUntil(".elem").parent().attr("id") == this.id) {
				window.foreui.focusedElementId = this.id;
				onPropertyValueChanged("FOCUSED_ELEMENT_ID", this.id);
			}
		});
	}
};

// Return ForeUI element by given id
var getForeUIElementById = function(id) {
	if (!window.foreui.elements[id]) {
		var constructor = window.foreui.elements.constructors[id];
		if (constructor) {
			constructor();
			// IE workaround
			if ($.browser.msie && id.indexOf("Table_") == 0) {
				changeVisibility([id], -1, false);
				changeVisibility([id], -1, false);
			}
			// init default behavior
			initDefaultBehaviorForElement(window.foreui.elements[id]);
			// init behavior
			initBehaviorForId(id);
			// trigger "Initialized" event
			window.foreui.elements[id]._obj.trigger("Initialized");
		}
	}
	return window.foreui.elements[id];
};

// Return the bounds object for object with given id
var getBoundsById = function(id) {
	return getBounds(document.getElementById(id));
};

// Return the bounds for given DOM object
var getBounds = function(obj) {
	var $obj = $(obj);
	var bounds = { x : obj.offsetLeft, y : obj.offsetTop, w : $obj.outerWidth(), h : $obj.outerHeight() };
	var bk = { x : obj.offsetLeft, y : obj.offsetTop};
	while((obj = obj.offsetParent) && !isPage(obj)) {
		bounds.x += obj.offsetLeft;
		bounds.y += obj.offsetTop;
	}
	bounds.parentX = bounds.x - bk.x;
	bounds.parentY = bounds.y - bk.y;
	return bounds;
};

// Return the total bounds for DOM objects with given ids, bounds for every object will be store in the given array
var getBoundsByIds = function(targets, bounds) {
	var minX, minY, maxX, maxY;
	for (var i = 0; i < targets.length; i ++) {
		bounds[i] = getBoundsById(targets[i]);
		if (minX == undefined || bounds[i].x < minX) {
			minX = bounds[i].x;
		}
		if (minY == undefined || bounds[i].y < minY) {
			minY = bounds[i].y;
		}
		if (maxX == undefined || bounds[i].x + bounds[i].w - 1 > maxX) {
			maxX = bounds[i].x + bounds[i].w - 1;
		}
		if (maxY == undefined || bounds[i].y + bounds[i].h - 1 > maxY) {
			maxY = bounds[i].y + bounds[i].h - 1;
		}
	}
	return { x : minX, y : minY, w : maxX - minX + 1, h : maxY - minY + 1 };
};

// Check if the element with childId is descendant of the element with parentId
var isDescendantOf = function(childId, parentId) {
	var child = $("#" + childId);
	var parent = $("#" + parentId);
	if (child.size() > 0 && parent.size() > 0) {
		return child.parents().index(parent) >= 0;
	}
	return false;
};

// Check if the element with childId is descendant of any element in specified with parentIds array
var isDescendantOfAny = function(childId, parentIds) {
	for (var i = 0; i < parentIds.length; i ++) {
		if (isDescendantOf(childId, parentIds[i])) {
			return true;
		}
	}
	return false;
};

// Check if the element with given id is an embedded element
var isEmbeddedElement = function(elemId) {
	return $("#" + elemId).parents(".elem").size() > 0;
};

// Append the ids of objects in target area, and cover the target objects
var appendInnerObjects = function(rc, targets) {
	if (targets.length > 0) {
		var page = getPageByObject(document.getElementById(targets[0]));
		var sortedTars = targets.slice();
		sortedTars.sort(function(a, b){
			return a == b ? 0 : (canACoverB(document.getElementById(a), document.getElementById(b)) ? 1 : -1);
		});
		var base = document.getElementById(sortedTars[0]);
		var inner = objectsInRect(page, rc.x, rc.y, rc.w, rc.h, base);
		for (var i = 0; i < inner.length; i ++) {
			if (!isEmbeddedElement(inner[i].id) && $.inArray(inner[i].id, targets) == -1 && !isDescendantOfAny(inner[i].id, targets)) {
				targets.push(inner[i].id);
			}
		}
	}
};

// Check if the element supports the given state
var supportState = function(elem, state) {
	return $.inArray(state, elem._params.states) != -1;
};

// Set the element state, if possible and needed
var setState = function(elem, state, force) {
	if (supportState(elem, state)) {
		if (force || (elem._params.state != state && (elem._params.state & 64) == 0)) {
			var wasSelected = (elem._params.state & 32);
			var isSelected = (state & 32);
			elem._params.state = state;
			if (elem.onStateChanged) {
				elem.onStateChanged();
			}
			if (elem.onSelected && !wasSelected && isSelected) {
				elem.onSelected();
			}
			if (elem.onUnselected && wasSelected && !isSelected) {
				elem.onUnselected();
			}
			updateElementAndReferences(elem);
		}
	}
};

// Get the current event object
var getEvent = function(){
	if(document.all) {
		return window.event;
	}
	func = getEvent.caller;
	while(func != null){
		var arg0 = func.arguments[0];
		if(arg0) {
			if((arg0.constructor==Event || arg0.constructor ==MouseEvent)
				|| (typeof(arg0)=="object" && arg0.preventDefault && arg0.stopPropagation)) {
				return arg0;
			}
		}
		func = func.caller;
	}
	return null;
};

// Convert the value by given type
var convertValue = function(value, type) {
	if (type == "N") {
		return parseFloat(value);
	} else if (type == "S") {
		if (isNumber(value)) {
			return "" + value;
		} else if ($.type(value) === "string") {
			return value;
		}
		return toJsonString(value);
	} else if (type == "A") {
		if ($.isArray(value)) {
			return value;
		}
		try {
			var json = $.parseJSON(value);
			if ($.isArray(json)) {
				return json;
			}
			return [json];
		} catch (e) {
			return [value];
		}
	} else if (type == "O") {
		if ($.isPlainObject(value)) {
			return value;
		}
		try {
			var json = $.parseJSON(value);
			if ($.isArray(json)) {
				return $.extend({}, json);
			} else if ($.isPlainObject(json)) {
				return json;
			}
			return {"value":value};
		} catch (e) {
			return {"value":value};
		}
	}
	return value;
}

// Get property value
var getProperty = function(name, type) {
	var dotPos = name.indexOf(".");
	if (dotPos != -1) {
		// element property
		var id = name.substring(0, dotPos);
		var propName = name.substring(dotPos + 1, name.length);
		return convertValue(getElementPropertyValue(id, propName), type);
	} else {
		var getter = window.foreui.sys_properties[name];
		if (getter != undefined) {
			// system property
			return convertValue(getter.call(), type);
		} else {
			var prop = window.foreui.properties[name];
			if (prop != null) {
				// custom property
				return convertValue(prop.value, type);
			}
		}
	}
	return null;
};

// Check if the given page is excluded
var isPageExcluded = function(pageIndex) {
	return $("#Page_" + pageIndex).attr("excluded") == "true";
};

// Return the element type according to the given id
var getElementTypeById = function(id) {
	var index = id.indexOf("_");
	if (index == -1) {
		return "Unknown";
	} else {
		return id.substring(0, index);
	}
}

// Return the id array for siblings of elements with given ids
var findSiblingsInGroup = function(targets, includeThemselves) {
	var ids = [];
	for (var i = 0; i < targets.length; i ++) {
		var elem = getForeUIElementById(targets[i]);
		if (elem) {
			if (includeThemselves && $.inArray(targets[i], ids) == -1) {
				ids.push(targets[i]);
			}
			var type = getElementTypeById(targets[i]);
			var parent = elem._obj.parent();
			if (parent.hasClass("elem") && parent.attr("id").indexOf("Group") == 0) {
				elem._obj.siblings(".elem").filter(function(index) {
					return (this.id && this.id.indexOf(type) == 0);
				}).each(function(index) {
					if ($.inArray(this.id, ids) == -1 && $.inArray(this.id, targets)) {
						ids.push(this.id);
					}
				});
			}
		}
	}
	return ids;
};

// Get the HTML code for viewing, according the input text
var htmlToView = function(text) {
	text = "" + text;
	var result = text.replace(/\n/gi, "<br>");
	result = result.replace(/([^\s])\s/gi, "$1&escsp;");
	result = result.replace(/\s/gi, "&nbsp;");
	result = result.replace(/&escsp;/gi, " ");
	return result;
};

// Get the text from the viewing html
var textFromView = function(html) {
	html = "" + html;
	var result = html.replace(/<br>/gi, "\n");
	result = result.replace(/&nbsp;/gi, " ");
	return result;
};

// Initialize toolbar
var initToolbar = function(conf) {
	if (conf.hasToolbar) {
		window.foreui.toolbar = new ForeUIToolbar(conf);
	}
};

// Convert value to JSON string, correctly display one based array and undefined value
var toJsonString = function(value) {
	if ($.isArray(value)) {
		value = oneBasedToZeroBased($.extend(true, [], value));
	} else if ($.isPlainObject(value)) {
		value = oneBasedToZeroBased($.extend(true, {}, value));
	}
	return value == undefined ? "" : $.toJSON(value);
};

// Update property observer for the change
var onPropertyValueChanged = function(name, value) {
	if (window.foreui.toolbar) {
		ForeUIToolbar.prototype.onPropertyChanged(name, value);
	}
};

// Preload image
var preloadImage = function(url) {
	var img = new Image();
	img.src = url;
};

// Check if passed parameter is a number
var isNumber = function(n) {
  return n===+n;
}

// Convert 0 based array to 1 based array
var zeroBasedToOneBased = function(value) {
	if ($.isArray(value)) {
		for (var i = 0; i < value.length; i ++) {
			value[i] = zeroBasedToOneBased(value[i]);
		}
		if (value[0] !== undefined) {
			value.splice(0, 0, undefined);
		}
	} else if ($.isPlainObject(value)) {
		for(var p in value) {
			if (value.hasOwnProperty(p)) {
				value[p] = zeroBasedToOneBased(value[p]);
			}
		}
	}
	return value;
};

// Convert 1 based array to 0 based array
var oneBasedToZeroBased = function(value) {
	if ($.isArray(value)) {
		for (var i = 0; i < value.length; i ++) {
			value[i] = oneBasedToZeroBased(value[i]);
		}
		value.splice(0, 1);
	} else if ($.isPlainObject(value)) {
		for(var p in value) {
			if (value.hasOwnProperty(p)) {
				value[p] = oneBasedToZeroBased(value[p]);
			}
		}
	}
	return value;
};

// Filter the invalid target element IDs
var filterTargets = function(targets) {
	if (!targets) {
		return targets;
	}
	var result = [];
	for (var i = 0; i < targets.length; i ++) {
		if (document.getElementById(targets[i])) {
			result.push(targets[i]);
		}
	}
	return result;
};

// Generate GUID
var generateGUID = function () {
	var rand4 = function () {
	    return Math.floor(Math.random() * 0x10000).toString(16);
	};
	return rand4() + rand4() + "-" + rand4() + "-" + rand4() + "-" + rand4() + "-" + rand4() + rand4() + rand4();
};

// Get lighter/darker color
function shadeColor(color, ratio) {
	if (color == 'transparent') {
		return color;
	}
    var f=parseInt(color.slice(1),16),t=ratio<0?0:255,p=ratio<0?ratio*-1:ratio,R=f>>16,G=f>>8&0x00FF,B=f&0x0000FF;
    return "#"+(0x1000000+(Math.round((t-R)*p)+R)*0x10000+(Math.round((t-G)*p)+G)*0x100+(Math.round((t-B)*p)+B)).toString(16).slice(1);
}

/******************************************************************************
 *	Some functions to get system property value
 *****************************************************************************/

// SCREEN_WIDTH
var getScreenWidth = function() {
	return document.body.clientWidth;
};

// SCREEN_HEIGHT
var getScreenHeight = function() {
	return document.body.clientHeight;
};

// CURRENT_PAGE_INDEX
function getCurrentPageIndex() {
	return window.foreui.curPage;
}

// CURRENT_PAGE_TITLE
function getCurrentPageTitle() {
	var pageDiv = document.getElementById("Page_" + getCurrentPageIndex());
	return pageDiv == null ? "" : pageDiv.getAttribute("pageTitle");
}

// AVAILABLE_PAGE_INDICES
function getAvailablePageIndices() {
	return window.foreui.pageIndices;
}

// AVAILABLE_PAGE_TITLES
function getAvailablePageTitles() {
	var titles = [undefined];
	var pageIndices = getAvailablePageIndices();
	for(var i = 1; i < pageIndices.length; i ++) {
		var pageDiv = document.getElementById("Page_" + parseInt(pageIndices[i]));
		if (pageDiv != null) {
			titles.push(pageDiv.getAttribute("pageTitle"));
		}
	}
	return titles;
}

// CURRENT_YEAR
function getCurrentYear() {
	return new Date().getFullYear();
}

// CURRENT_MONTH
function getCurrentMonth() {
	var months = ['January','February','March','April','May','June','July','August','September','October','November','December'];
	var shortMonths = ['Jan','Feb','Mar','Apr','May','Jun','Jul','Aug','Sep','Oct','Nov','Dec'];
	var now = new Date();
	var index = now.getMonth();
	return {
		'number' : index + 1,
		'full' : months[index],
		'short' : shortMonths[index]
	};
}

// CURRENT_DAY
function getCurrentDay() {
	return new Date().getDate();
}

// CURRENT_DAY_OF_WEEK
function getCurrentDayOfWeek() {
	var days = ['Sunday','Monday','Tuesday','Wednesday','Thursday','Friday','Saturday'];
	var shortDays = ['Sun','Mon','Tue','Wed','Thu','Fri','Sat'];
	var now = new Date();
	var index = now.getDay();
	return {
		'index' : index,
		'full' : days[index],
		'short' : shortDays[index]
	};
}

// CURRENT_HOURS
function getCurrentHours() {
	return new Date().getHours();
}

// CURRENT_MINUTES
function getCurrentMinutes() {
	return new Date().getMinutes();
}

// CURRENT_SECONDS
function getCurrentSeconds() {
	return new Date().getSeconds();
}

// CURRENT_TIMESTAMP
function getCurrentTimestamp() {
	return new Date().getTime();
}

// CURRENT_KEY_CODE
function getCurrentKeyCode() {
	var event = getEvent();
	return event == null ? window.foreui.cache.keyCode : event.keyCode;
}

// CTRL_KEY_STATE
function getCtrlKeyState() {
	var event = getEvent();
	return (event != null && event.ctrlKey == true) ? 1 : window.foreui.cache.ctrlState;
}

// ALT_KEY_STATE
function getAltKeyState() {
	var event = getEvent();
	return (event != null && event.altKey == true) ? 1 : window.foreui.cache.altState;
}

// SHIFT_KEY_STATE
function getShiftKeyState() {
	var event = getEvent();
	return (event != null && event.shiftKey == true) ? 1 : window.foreui.cache.shiftState;
}

// FOCUSED_ELEMENT_ID
function getFocusedElementId() {
	return window.foreui.focusedElementId;
}

// CURRENT_CURSOR_X
function getCurrentCursorX() {
	var event = getEvent();
	if (event != null) {
		return event.clientX;
	}
	return -1;
}

// CURRENT_CURSOR_Y
function getCurrentCursorY() {
	var event = getEvent();
	if (event != null) {
		return event.clientY;
	}
	return -1;
}

// CURRENT_EVENT
function getCurrentEvent() {
	return window.foreui.currentEvent;
}

/******************************************************************************
 *	Function to get element property value
 *****************************************************************************/
var getElementPropertyValue = function(id, name) {
	switch (name) {
	case "id":
		return id;
	case "x":
		return getBoundsById(id).x;
	case "y":
		return getBoundsById(id).y;
	case "cx":
		var rc = getBoundsById(id);
		return rc.x + rc.w / 2;
	case "cy":
		var rc = getBoundsById(id);
		return rc.y + rc.h / 2;
	case "width":
		return getBoundsById(id).w;
	case "height":
		return getBoundsById(id).h;
	case "visible":
		var elem = getForeUIElementById(id);
		if (elem) {
			return elem._obj.css("visibility") != "hidden";
		}
		return false;
	case "note":
		var elem = getForeUIElementById(id);
		if (elem) {
			var note = elem._params.note;
			return (note == null) ? "" : note;
		}
		return "";
	default:
		var elem = getForeUIElementById(id);
		if (elem) {
			var getter = elem["get" + name.substring(0, 1).toUpperCase() + name.substring(1, name.length)];
			if (getter != undefined) {
				return getter.call(elem);
			}
		}
	}
	return null;
};

/******************************************************************************
 *	Some functions to execute actions
 *****************************************************************************/

// Switch to page with given index
var goToPage = function(pageIndex, callback, slideDirection) {
	if (window.foreui.curPage == pageIndex) {
		if (callback) {
			callback();
		}
		return;
	}
	if (window.foreui.lazy) {
		// Show the loading indicator
		$("#loading").show();
	}
	setTimeout(function() {
		var task = function() {
			var pages = $(".page").css("visibility", "hidden");
			// restore elements from master pages
			var targetPage = $("#Page_" + pageIndex);
			if (window.foreui.curPage) {
				var restoreMap = {};
				$("#Page_" + window.foreui.curPage).children(".fromMaster").each(function() {
					var elem = $(this).removeClass("fromMaster");
					var srcPageId = elem.attr("srcPage");
					elem.removeAttr("srcPage");
					if (restoreMap[srcPageId] == undefined) {
						restoreMap[srcPageId] = $("<div>/</div>");
					}
					restoreMap[srcPageId].append(elem);
				});
				for (var srcPageId in restoreMap) {
					$("#" + srcPageId).append(restoreMap[srcPageId].children());
				}
			}
			window.foreui.curPage = pageIndex;
			// move elements from master page to target page
			var masterPages = targetPage.attr("masterPages").split(",");
			var masterContent = $("<div></div>");
			var masterPageList = [];
			for (var i = 0; i < masterPages.length; i ++) {
				if (masterPages[i] != "") {
					var masterPage = $("#" + masterPages[i]);
					masterPage.children(".elem")
						.addClass("fromMaster")
						.attr("srcPage", masterPages[i])
						.appendTo(masterContent);
					masterPageList.push(masterPage);
				}
			}
			masterContent.children().prependTo(targetPage);
			if (window.foreui.lazy) {
				targetPage.find(".elem").each(function() {
					getForeUIElementById($(this).attr("id"));
				});
			}
			// show target page
			targetPage.css("visibility", "visible")
				.position({
					my : window.foreui.align + " top",
					at : window.foreui.align + " top",
					of : document
				}).css("top", "0px");
			if (slideDirection) {
				var dir = (slideDirection == "left" ? "right" : "left");
				dir = (slideDirection == "up" ? "down" : (slideDirection == "down" ? "up" : dir));
				targetPage.css("display", "none")
					.show("slide", { "direction" : dir }, 300);
			}
			// trigger "Load as Master" and "Page Loaded" events
			for (var i = 0; i < masterPageList.length; i ++) {
				masterPageList[i].trigger("Loaded_As_Master");
			}
			targetPage.trigger("Page_Loaded");
			// update system properties
			onPropertyValueChanged("CURRENT_PAGE_INDEX", pageIndex);
			onPropertyValueChanged("CURRENT_PAGE_TITLE", getCurrentPageTitle());
			// update page indicator
			var pageCount = pages.size();
			var actualPageCount = 0;
			var actualIndex = 0;
			window.foreui.pageIndices = [undefined];
			for (var i = 1; i <= pageCount; i ++) {
				if (!isPageExcluded(i)) {
					actualPageCount ++;
					window.foreui.pageIndices.push(i);
				}
				if (actualIndex == 0 && i == pageIndex) {
					actualIndex = actualPageCount;
				}
			}
			setTimeout(function() {
				if (window.foreui.toolbar) {
					ForeUIToolbar.prototype.updatePageIndicator(actualIndex, actualPageCount);
					ForeUIToolbar.prototype.updateSelectedPage(pageIndex);
				}
				// Hide the loading indicator
				$("#loading").hide();
			}, 50);
			onPropertyValueChanged("AVAILABLE_PAGE_INDICES", window.foreui.pageIndices);
			onPropertyValueChanged("AVAILABLE_PAGE_TITLES", getAvailablePageTitles());
			if (callback) {
				callback();
			}
		};
		// hide all pages
		var curPage = $("#Page_" + window.foreui.curPage);
		curPage.trigger("Page_Unloaded");
		if (slideDirection && curPage.length > 0) {
			curPage.hide("slide", { "direction" : slideDirection }, 300, function() {
				curPage.css("display", "block");
				task();
			});
		} else {
			task();
		}
	}, 0);
}

// Switch to previous page
var previousPage = function(slideDirection) {
	var curPage = getCurrentPageIndex();
	var targetPage = curPage - 1;
	while (isPageExcluded(targetPage)) {
		targetPage -= 1;
	}
	if (targetPage >= 1) {
		goToPage(targetPage, undefined, slideDirection);
	}
};

// Switch to next page
var nextPage = function(slideDirection) {
	var curPage = getCurrentPageIndex();
	var pageCount = $(".page").size();
	var targetPage = curPage + 1;
	while (isPageExcluded(targetPage)) {
		targetPage += 1;
	}
	if (targetPage <= pageCount) {
		goToPage(targetPage, undefined, slideDirection);
	}
};

// Set cursor style
var setGlobalCursor = function(cursor) {
	$("body").css("cursor", cursor);
};

// Change the visibility of objects with given ids, with or without the content inside.
var changeVisibility = function(targets, visible, withInner, duration) {
	var targets = filterTargets(targets);
	var rc = getBoundsByIds(targets, []);
	if (withInner) {
		appendInnerObjects(rc, targets);
	}
	for (var i = 0; i < targets.length; i ++) {
		var $obj = getForeUIElementById(targets[i])._obj;
		var obj = $obj.get(0);
		var newState = visible == 0 ? "hidden" : (visible == 1 ? "inherit" : (obj.style.visibility == "inherit" ? "hidden" : "inherit"));
		if (duration) {
			$obj.data("newState", newState);
			if (newState != "hidden") {
				$obj.css({
					opacity : 0,
					visibility : newState
				});
			}
			$obj.animate({
					opacity : newState == "hidden" ? 0 : 1
				}, duration, function() {
					$obj = $(this)
					$obj.css({
						opacity : 1,
						visibility : $obj.data("newState")
					}).removeData("newState");
				});
		} else {
			$obj.css("visibility", newState);
		}
		$obj.trigger(newState == "hidden" ? "Hidden" : "Visible");
	}
};

// Change the state of objects with given ids, with or without the content inside.
var changeState = function(targets, state, withInner) {
	var targets = filterTargets(targets);
	var rc = getBoundsByIds(targets, []);
	if (withInner) {
		appendInnerObjects(rc, targets);
	}
	for (var i = 0; i < targets.length; i ++) {
		var elem = getForeUIElementById(targets[i]);
		if (elem) {
			setState(elem, state, true);
		}
	}
};

// Move the objects with given ids to (x,y), with or without the content inside, related positioning will be kept.
var move = function(targets, x, y, withInner, duration) {
	var targets = filterTargets(targets);
	var xHead = (x + "  ").substring(0, 2);
	var yHead = (y + "  ").substring(0, 2);
	var xSign = xHead == "+=" ? 1 : (xHead == "-=" ? -1 : 0);
	var ySign = yHead == "+=" ? 1 : (yHead == "-=" ? -1 : 0);
	var newX = parseFloat(xSign == 0 ? x : eval(x.substring(2)));
	var newY = parseFloat(ySign == 0 ? y : eval(y.substring(2)));
	var bounds = [];
	var rc = getBoundsByIds(targets, bounds);
	if (withInner) {
		appendInnerObjects(rc, targets);
	}
	for (var i = 0; i < targets.length; i ++) {
		if (bounds[i] == undefined) {
			bounds[i] = getBoundsById(targets[i]);
		}
		var elem = getForeUIElementById(targets[i]);
		var obj = elem._obj.get(0);
		var tarX = (xSign == 0 ? newX + bounds[i].x - rc.x - bounds[i].parentX : parseInt(obj.style.left) + xSign * newX) + "px";
		var tarY = (ySign == 0 ? newY + bounds[i].y - rc.y - bounds[i].parentY : parseInt(obj.style.top) + ySign * newY) + "px";
		var cssObj = { left : tarX, top : tarY };
		if (duration) {
			elem._obj.animate(cssObj, duration);
		} else {
			elem._obj.css(cssObj);
		}
	}
};

// Resize the objects with given ids, with or without the content inside, related sizing will be kept.
var resize = function(targets, w, h, withInner, duration) {
	var targets = filterTargets(targets);
	var wHead = (w + "  ").substring(0, 2);
	var hHead = (h + "  ").substring(0, 2);
	var wSign = wHead == "+=" ? 1 : (wHead == "-=" ? -1 : 0);
	var hSign = hHead == "+=" ? 1 : (hHead == "-=" ? -1 : 0);
	var newW = parseFloat(wSign == 0 ? w : eval(w.substring(2)));
	var newH = parseFloat(hSign == 0 ? h : eval(h.substring(2)));
	var bounds = [];
	var rc = getBoundsByIds(targets, bounds);
	if (withInner) {
		appendInnerObjects(rc, targets);
	}
	for (var i = 0; i < targets.length; i ++) {
		if (bounds[i] == undefined) {
			bounds[i] = getBoundsById(targets[i]);
		}
		var x, y, width, height;
		if (wSign == 0) {
			x = rc.x + (bounds[i].x - rc.x) * newW / rc.w;
			width = newW * bounds[i].w / rc.w;
		} else {
			width = bounds[i].w + wSign * newW * bounds[i].w / rc.w;
			x = bounds[i].x + wSign * (bounds[i].x - rc.x) * newW / rc.w;
		}
		if (hSign == 0) {
			y = rc.y + (bounds[i].y - rc.y) * newH / rc.h;
			height = newH * bounds[i].h / rc.h;
		} else {
			height = bounds[i].h + hSign * newH * bounds[i].h / rc.h;
			y = bounds[i].y + hSign * (bounds[i].y - rc.y) * newH / rc.h;
		}
		var cssObj = {
			width : width + "px",
			height : height + "px"
		};
		if (x != bounds[i].x) {
			cssObj["left"] = x + "px";
		}
		if (y != bounds[i].y) {
			cssObj["top"] = y + "px";
		}
		var elem = getForeUIElementById(targets[i]);
		if (duration) {
			elem._obj.animate(cssObj, {
				duration : duration,
				step : function(now, fx){
					updateElementAndReferences(getForeUIElementById(fx.elem.id));
				}
			});
		} else {
			elem._obj.css(cssObj);
			updateElementAndReferences(elem);
		}
	}
};

// Change background color for target elements
var changeBgColor = function(targets, color) {
	var targets = filterTargets(targets);
	for (var i = 0; i < targets.length; i ++) {
		var elem = getForeUIElementById(targets[i]);
		if (elem) {
			if (elem.setBgColor) {
				elem.setBgColor(color);
				updateElementAndReferences(elem);
			}
		}
	}
}

// Change background opacity for target elements
var changeOpacity = function(targets, opacity) {
	var targets = filterTargets(targets);
	for (var i = 0; i < targets.length; i ++) {
		var elem = getForeUIElementById(targets[i]);
		if (elem) {
			if (elem.setBgOpacity) {
				elem.setBgOpacity(opacity);
				updateElementAndReferences(elem);
			}
		}
	}
}

// Change text color for target elements
var changeTextColor = function(targets, color) {
	var targets = filterTargets(targets);
	for (var i = 0; i < targets.length; i ++) {
		var elem = getForeUIElementById(targets[i]);
		if (elem) {
			if (elem.setTextColor) {
				elem.setTextColor(color);
				updateElementAndReferences(elem);
			}
		}
	}
}

// Trigger the custom event with given name, maybe on specific targets
var triggerEvent = function(targets, eventName) {
	var targets = filterTargets(targets);
	if (targets) {
		for (var i = 0; i < targets.length; i ++) {
			getForeUIElementById(targets[i])._obj
				.bind(eventName, {}, false)
				.trigger(eventName);
		}
	} else {
		$(document).trigger(eventName);
	}
};

// Set user defined property
var setCustomProperty = function(name, value, type) {
	if (type == 0) {
		if (!isNumber(value)) {
			var fv = parseFloat(value);
			var iv = parseInt(value);
			value = (fv == iv) ? iv : fv;
		}
	} else if (type == 2) {
		if (!$.isArray(value)) {
			value = [value];
		}
	}
	value = zeroBasedToOneBased(value);
	window.foreui.properties[name] = {
		"name" : name,
		"value" : value,
		"type" : type
	};
	onPropertyValueChanged(name, value);
};

// Set selected index on target elements with given id
var setSelectedIndex = function(targets, index) {
	var targets = filterTargets(targets);
	for (var i = 0; i < targets.length; i ++) {
		getForeUIElementById(targets[i]).setSelectedIndex(index);
	}
};

// Get selected index of element with given id
var getSelectedIndex = function(target) {
	return getForeUIElementById(target).getSelectedIndex();
};

// Set selected text on target elements with given id
var setSelectedText = function(targets, text) {
	var targets = filterTargets(targets);
	for (var i = 0; i < targets.length; i ++) {
		getForeUIElementById(targets[i]).setSelectedText(text);
	}
};

// Get selected text of element with given id
var getSelectedText = function(target) {
	return getForeUIElementById(target).getSelectedText();
};

// Set current value on target elements with given id
var setCurrentValue = function(targets, value) {
	var targets = filterTargets(targets);
	for (var i = 0; i < targets.length; i ++) {
		var elem = getForeUIElementById(targets[i]);
		if (elem && elem.setCurrentValue) {
			elem.setCurrentValue(value);
		}
	}
};

// Get current value of element with given id
var getCurrentValue = function(target) {
	var elem = getForeUIElementById(target);
	if (elem && elem.getCurrentValue) {
		return elem.getCurrentValue();
	}
	return undefined;
};

// Set local storage data
var setLocalStorageData = function(targets, dataKey, dataVal) {
	for (var i = 0; i < targets.length; i ++) {
		var elem = getForeUIElementById(targets[i]);
		if (elem && elem.setDataByKey) {
			elem.setDataByKey(dataKey, dataVal);
		}
	}
}

// Get data from local storage
var getLocalStorageData = function(target, datakey) {
	var elem = getForeUIElementById(target);
	if (elem && elem.getDataByKey) {
		return elem.getDataByKey(dataKey);
	}
	return undefined;
};

// Set target of Reference elements
var setReferenceTarget = function(targets, targetId) {
	var targets = filterTargets(targets);
	for (var i = 0; i < targets.length; i ++) {
		var elem = getForeUIElementById(targets[i]);
		if (elem && elem.setReferenceTarget) {
			elem.setReferenceTarget(targetId);
		}
	}
};

// Set focus on target elements with given id
var setFocus = function(targets) {
	var targets = filterTargets(targets);
	for (var i = 0; i < targets.length; i ++) {
		getForeUIElementById(targets[i]).setFocus();
	}
}

// Set image source on target elements with given id
var setImageSource = function(targets, value) {
	var targets = filterTargets(targets);
	var m = value.match(/\d+/);
	if (m != null) {
		var fileName = m[0] + "." + window.foreui.imagetypes[m[0]];
		for (var i = 0; i < targets.length; i ++) {
			getForeUIElementById(targets[i]).setImageSource(fileName);
		}
	}
};

// Set current handling event
var setCurrentEvent = function(ev) {
	window.foreui.currentEvent = ev;
	onPropertyValueChanged("CURRENT_EVENT", ev);
};

// Set tooltip on target elements with given id
var setTooltip = function(targets, tooltip) {
	var targets = filterTargets(targets);
	for (var i = 0; i < targets.length; i ++) {
		var elem = getForeUIElementById(targets[i]);
		elem._params.tooltip = tooltip;
		if (tooltip != "") {
			$("#" + elem._id + ", [ref=\"" + elem._id + "\"]").attr("title", tooltip);
		} else {
			$("#" + elem._id + ", [ref=\"" + elem._id + "\"]").removeAttr("title");
		}
	}
};

// Apply box shadow to given jQuery block object
var applyShadow = function(elemObj, bgObj, shadow) {
	if (shadow) {
		elemObj.css("overflow", "visible");
		bgObj.parentsUntil(elemObj).css("overflow", "visible");
		bgObj.css({
			"webkit-box-shadow":shadow,
			"-moz-box-shadow":shadow,
			"box-shadow":shadow
		});
	}
};


// Apply shadow to given jQuery text object
var applyTextShadow = function(textObj, params) {
	if (params && params.shadow) {
		textObj.css("text-shadow", params.shadowOffsetX + "px " + params.shadowOffsetY + "px " + params.shadowBlur + "px " + params.shadowColor);
	}
};


// Apply drop shadow to given jQuery object
var applyDropShadow = function(elemObj, imgObj, params) {
	var shadow = params.shadow;
	if (shadow) {
		elemObj.css("overflow", "visible");
		imgObj.parentsUntil(elemObj).css("overflow", "visible");
		var dropShadow = "drop-shadow("+ params.shadowOffsetX + "px " + params.shadowOffsetY + "px " + params.shadowBlur + "px " + params.shadowColor + ")";
		var dsFilter = "url(\"data:image/svg+xml;utf8,<svg height='0' xmlns='http://www.w3.org/2000/svg'><filter id='drop-shadow'><feGaussianBlur in='SourceAlpha' stdDeviation='" + params.shadowBlur + "'/><feOffset dx='" + params.shadowOffsetX + "' dy='" + params.shadowOffsetY + "' result='offsetblur'/><feFlood flood-color='" + params.shadowColor + "'/><feComposite in2='offsetblur' operator='in'/><feMerge><feMergeNode/><feMergeNode in='SourceGraphic'/></feMerge></filter></svg>#drop-shadow\")";
		imgObj.css({
			"-webkit-filter":dropShadow,
			"filter":dsFilter
		});
	}
}