var ForeUIToolbar = function(conf) {
	this._conf = conf;
	this._init();
};

ForeUIToolbar.prototype = {
	_init : function() {
		var conf = this._conf;
		var body = $("body");
		this._toolbar = $("<div class=\"toolbar\"></div>").appendTo(body);
		this._controls = $("<div class=\"controls\" style=\"white-space:nowrap;\"></div>").appendTo(this._toolbar);
		this._controls.append("<img src=\"styles/foreui_logo.gif\" align=\"absmiddle\" />");
		this._controls.append("<span style=\"width:10px;\"></span>");
		this._pageLabel = $("<span>Page:</span>").appendTo(this._controls);
		this._prevButton = $("<span class=\"button\" title=\"Go to previous page (Page Up)\" style=\"width:14px\">&lt;</span>").appendTo(this._controls);
		this._pageIndicator = $("<span class=\"pageIndicator\">1/1</span>").appendTo(this._controls);
		this._nextButton = $("<span class=\"button\" title=\"Go to next page (Page Down)\" style=\"width:14px\">&gt;</span>").appendTo(this._controls);
		var maxHeight = document.body.clientHeight * 0.8;
		
		// page list
		this._gotoButton = $("<span class=\"button\" title=\"Choose a page from the list\">Goto...</span>").appendTo(this._controls);
		this._pageList = $("<div class=\"pageList winFrame\"><div class=\"winTitle\"><div class=\"winTitleClose\">X</div>Pages:</div><div class=\"winClient\"><ul></ul></div></div>").appendTo(body);
		var pgList = this._pageList.children().children("ul");
		var pgTreeNodes = conf.pageTree.children;
		for (var i = 0; i < pgTreeNodes.length; i ++) {
			this._appendPgTreeNode(pgTreeNodes[i], pgList);
		}
		this._pageList.width(Math.min(400, Math.max(200, this._pageList.width() + 20)));
		this._pageList.height(Math.min(maxHeight, Math.max(200, this._pageList.height() + 20)));
		
		// property list
		this._controls.append("<span style=\"width:10px;\"></span>");
		this._propertyButton = $("<span class=\"button\" title=\"View Properties\">Properties</span>").appendTo(this._controls);
		this._propertyList = $("<div class=\"propertyList winFrame\"><div class=\"winTitle\"><div class=\"winTitleClose\">X</div>Properties:</div><div class=\"winClient\"><table border=\"1\" cellspacing=\"0\" cellpadding=\"5\"></table></div></div>").appendTo(body);
		var propTable = this._propertyList.children().children("table");
		var sysProp = window.foreui.sys_properties;
		for (propName in sysProp) {
			var row = $("<tr></tr>").appendTo(propTable);
			$("<td style=\"font-weight:bold;\"></td>").append(propName).appendTo(row);
			$("<td style=\"font-weight:bold;\" width=\"50%\" class=\"" + propName + "\">" + toJsonString(getProperty(propName)) + "</td>").appendTo(row);
		}
		var customProp = window.foreui.properties;
		for (propName in customProp) {
			var row = $("<tr></tr>").appendTo(propTable);
			$("<td></td>").append(propName).appendTo(row);
			$("<td width=\"50%\" class=\"" + propName + "\">" + toJsonString(getProperty(propName)) + "</td>").appendTo(row);
		}
		this._propertyList.width(Math.min(400, Math.max(300, this._propertyList.width())));
		this._propertyList.height(Math.min(maxHeight, Math.max(300, this._propertyList.height())));
		this._schedulePropertyUpdate(propTable);
		
		// content alignment
		this._controls.append("<span style=\"width:10px;\"></span>");
		this._alignButton = $("<span class=\"button\" title=\"Click to Change Content Alignment\">Content Align: " + window.foreui.align + "</span>").appendTo(this._controls);
		
		this._pinButton = $("<img class=\"pinButton\" src=\"styles/" + (conf.pin ? "pin.gif" : "unpin.gif") + "\" title=\"Click to pin/unpin the toolbar\" style=\"position:absolute;cursor:pointer;\">").appendTo(this._toolbar);
		
		this._dockButtons = $("<div class=\"dockButtons\"></div>").appendTo(this._toolbar);
		this._dockTop = $("<div class=\"dockButton dockTop\" title=\"Dock to top\">").appendTo(this._dockButtons);
		this._dockLeft = $("<div class=\"dockButton dockLeft\" title=\"Dock to left\">").appendTo(this._dockButtons);
		this._dockBottom = $("<div class=\"dockButton dockBottom\" title=\"Dock to bottom\">").appendTo(this._dockButtons);
		this._dockRight = $("<div class=\"dockButton dockRight\" title=\"Dock to right\">").appendTo(this._dockButtons);
		this["_dock" + conf.dockTo.charAt(0).toUpperCase() + conf.dockTo.substr(1)].addClass("hlDockButton");
		
		this._setupBehavior();
		var me = this;
		setTimeout(function() {
			me._updateLayout();
			me._toolbar.trigger("mouseout");
		}, 50);
	},
	_appendPgTreeNode : function(node, obj) {
		var children = node.children;
		var actualIndex = parseInt(node.index) + 1;
		var li = $("<li></li>").attr({
			"type" : node.type,
			"excluded" : node.excluded
		}).appendTo(obj);
		if (node.type == "page") {
			li.attr("pageIndex", actualIndex);
		}
		var nodeContent = $(node.excluded || node.type == "folder" ? "<span></span>" : "<a href=\"javascript:goToPage(" + actualIndex + ");\"></a>")
			.append(node.title).appendTo(li);
		var icon = nodeContent.children("img");
		if (node.type == "folder") {
			li.css("list-style", "url(\"styles/" + (node.excluded ? "folder_excluded.png" : "folder.png") + "\")");
		} else {
			li.css("list-style", "url(\"styles/" + (node.excluded ? "page_excluded.png" : "page.png") + "\")");
		}
		if (node.excluded) {
			li.children("span").addClass("excluded");
		}
		if (children.length > 0) {
			var ul = $("<ul></ul>").appendTo(li);
			for (var i = 0; i < children.length; i ++) {
				this._appendPgTreeNode(children[i], ul);
			}
		}
	},
	_updateLayout : function(recall) {
		var conf = this._conf;
		var controls = this._controls;
		var pos, width, height, bg, pinPos, pinOffset, dockPos, pinBase, dockOffset;
		switch (conf.dockTo) {
			case "top":
			case "bottom":
				pos = "left " + conf.dockTo;
				width = "100%";
				height = "30px";
				bg = "url(\"styles/toolbar_hor.png\") repeat-x";
				pinPos = "right center";
				pinOffset = "-5 0";
				dockPos = "right center";
				pinBase = "left center";
				dockOffset = "-10 0";
				controls.removeClass("rotate90Degrees");
			break;
			case "left":
			case "right":
				pos = conf.dockTo + " top";
				width = "30px";
				height = "100%";
				bg = "url(\"styles/toolbar_vert.png\") repeat-y";
				pinPos = "center bottom";
				pinOffset = "0 -5";
				dockPos = "center bottom";
				pinBase = "center top";
				dockOffset = "0 -10";
				controls.addClass("rotate90Degrees");
			break;
		}
		this._toolbar.css({
			"width" : width,
			"height" : height,
			"background" : bg,
			"margin" : "0px"
		}).position({
			"my" : pos,
			"at" : pos,
			"of" : window
		});
		this._pinButton.position({
			"my" : pinPos,
			"at" : pinPos,
			"of" : this._toolbar,
			"offset" : pinOffset
		});
		this._dockButtons.position({
			"my" : dockPos,
			"at" : pinBase,
			"of" : this._pinButton,
			"offset" : dockOffset
		});
		if (recall) {
			this._updateLayout(false);
		}
	},
	_setupBehavior : function(obj) {
		var conf = this._conf;
		var me = this;
		// show/hide buttons
		if (!conf.pageButton) {
			this._pageLabel.add(this._prevButton).add(this._pageIndicator).add(this._nextButton).add(this._gotoButton).hide();
		}
		if (!conf.propertyButton) {
			this._propertyButton.hide();
		}
		if (!conf.alignButton) {
			this._alignButton.hide();
		}
		// dock to a side
		$(window).bind("scroll resize", function() {
			me._updateLayout();
			me._toolbar.trigger("mouseout");
		});
		// pin/unpin
		this._pinButton.click(function() {
			conf.pin = !conf.pin;
			me._pinButton.attr("src", "styles/" + (conf.pin ? "pin.gif" : "unpin.gif"));
		});
		var hideTimer;
		this._toolbar.mouseover(function() {
			if (hideTimer) {
				clearTimeout(hideTimer);
				hideTimer = undefined;
			}
			me._toolbar.css("margin", "0px");
			me._controls.css("visibility", "inherit");
			switch (conf.dockTo) {
				case "bottom":
					me._toolbar.css("height", "30px");
				break;
				case "right":
					me._toolbar.css("width", "30px");
				break;
			}
		}).mouseout(function() {
			if (!conf.pin) {
				if (hideTimer) {
					clearTimeout(hideTimer);
				}
				hideTimer = setTimeout(function() {
					switch (conf.dockTo) {
						case "top":
							me._toolbar.animate({ marginTop : "-28px" }, 1000);
						break;
						case "bottom":
							me._toolbar.animate({ marginTop : "28px", height : "2px" }, 1000);
						break;
						case "left":
							me._toolbar.animate({ marginLeft : "-28px" }, 1000);
						break;
						case "right":
							me._toolbar.animate({ marginLeft : "28px", width : "2px" }, 1000, function() {
								me._controls.css("visibility", "hidden");
							});
						break;
					}
				}, 2000);
			}
		});
		// dock to any side
		var dockButtons = this._dockButtons.children(".dockButton");
		dockButtons.click(function() {
			dockButtons.removeClass("hlDockButton");
			$(this).addClass("hlDockButton");
		});
		this._dockTop.click(function() {
			conf.dockTo = "top";
			me._updateLayout(true);
		});
		this._dockLeft.click(function() {
			conf.dockTo = "left";
			me._updateLayout(true);
		});
		this._dockBottom.click(function() {
			conf.dockTo = "bottom";
			me._updateLayout(true);
		});
		this._dockRight.click(function() {
			conf.dockTo = "right";
			me._updateLayout(true);
		});
		// page navigation
		this._prevButton.click(function() { previousPage(); });
		this._nextButton.click(function() { nextPage(); });
		this._gotoButton.click(function() {
			if (me._propertyList.is(":visible")) {
				me._toggleWindow(me._propertyList, me._propertyButton);
			}
			me._toggleWindow(me._pageList, me._gotoButton);
			me._updateLayout();
		});
		// property view
		this._propertyButton.click(function() {
			if (me._pageList.is(":visible")) {
				me._toggleWindow(me._pageList, me._propertyButton);
			}
			me._toggleWindow(me._propertyList, me._propertyButton);
			me._updateLayout();
		});
		// content alignment
		this._alignButton.click(function() {
			window.foreui.align = window.foreui.align == "left" ? "center" : (window.foreui.align == "center" ? "right" : "left");
			$(this).html("Content Align: " + window.foreui.align);
			$(".page").position({
				my : window.foreui.align + " top",
				at : window.foreui.align + " top",
				of : document
			}).css("top", "0px");
		});
		// window behavior
		$("body").children(".winFrame").resizable({
			"handles" : "all",
			"minWidth" : 100,
			"minHeight" : 30
		}).draggable({
			"handle" : ".winTitle"
		}).resize(function() {
			var win = $(this);
			win.children(".winClient").width(win.width()).height(win.height() - 30);
		}).children(".winTitle").children(".winTitleClose").click(function() {
			$(this).closest(".winFrame").css({
				"visibility" : "hidden",
				"display" : "none"
			});
		});
	},
	_toggleWindow : function(win, trigger) {
		var me = this;
		var conf = this._conf;
		win.css("display", win.css("visibility") == "hidden" ? "block" : "none");
		win.css("visibility", win.css("visibility") == "hidden" ? "inherit" : "hidden");
		
		var maxHeight = document.body.clientHeight * 0.8;
		win.width(Math.min(400, Math.max(200, win.width() + 20)));
		win.height(Math.min(maxHeight, Math.max(200, win.height() + 20)));
		win.trigger("resize");
		
		switch (conf.dockTo) {
			case "top":
				win.position({
					"my" : "left top",
					"at" : "left bottom",
					"of" : trigger ? trigger : me._toolbar
				});
			break;
			case "bottom":
				win.position({
					"my" : "left bottom",
					"at" : "left top",
					"of" : trigger ? trigger : me._toolbar
				});
			break;
			case "left":
				win.position({
					"my" : "left center",
					"at" : "right center",
					"of" : me._toolbar
				});
			break;
			case "right":
				win.position({
					"my" : "right center",
					"at" : "left center",
					"of" : me._toolbar
				});
			break;
		}
	},
	_schedulePropertyUpdate : function(propTable) {
		setInterval(function() {
			if (propTable.is(":visible")) {
				propTable.find(".CURRENT_YEAR").html(getProperty("CURRENT_YEAR"));
				propTable.find(".CURRENT_MONTH").html(toJsonString(getProperty("CURRENT_MONTH", "O")));
				propTable.find(".CURRENT_DAY").html(getProperty("CURRENT_DAY"));
				propTable.find(".CURRENT_DAY_OF_WEEK").html(toJsonString(getProperty("CURRENT_DAY_OF_WEEK", "O")));
				propTable.find(".CURRENT_HOURS").html(getProperty("CURRENT_HOURS"));
				propTable.find(".CURRENT_MINUTES").html(getProperty("CURRENT_MINUTES"));
				propTable.find(".CURRENT_SECONDS").html(getProperty("CURRENT_SECONDS"));
				propTable.find(".CURRENT_TIMESTAMP").html(getProperty("CURRENT_TIMESTAMP"));
			}
		}, 1000);
		$(window).resize(function() {
			propTable.find(".SCREEN_WIDTH").html(getScreenWidth());
			propTable.find(".SCREEN_HEIGHT").html(getScreenHeight());
		});
		$(document).mousemove(function(event) {
			propTable.find(".CURRENT_CURSOR_X").html(event.pageX);
			propTable.find(".CURRENT_CURSOR_Y").html(event.pageY);
		}).bind("keydown", function() {
			propTable.find(".CURRENT_KEY_CODE").html(getCurrentKeyCode());
			propTable.find(".CTRL_KEY_STATE").html(getCtrlKeyState());
			propTable.find(".ALT_KEY_STATE").html(getAltKeyState());
			propTable.find(".SHIFT_KEY_STATE").html(getShiftKeyState());
		}).keyup(function() {
			propTable.find(".CURRENT_KEY_CODE").html("-1");
			propTable.find(".CTRL_KEY_STATE").html(getCtrlKeyState());
			propTable.find(".ALT_KEY_STATE").html(getAltKeyState());
			propTable.find(".SHIFT_KEY_STATE").html(getShiftKeyState());
		});
	},
	onPropertyChanged : function(name, value) {
		var propTable = $("body").children(".propertyList").find("table");
		if (propTable.size() > 0) {
			var propCell = propTable.find("." + name);
			if (propCell.size() > 0) {
				propCell.html(toJsonString(value));
			} else {
				var row = $("<tr></tr>").appendTo(propTable);
				$("<td></td>").append(name).appendTo(row);
				$("<td width=\"50%\" class=\"" + name + "\">" + toJsonString(value) + "</td>").appendTo(row);
			}
		}
	},
	updatePageIndicator : function(actualIndex, actualPageCount) {
		$("body").children(".toolbar").find(".pageIndicator").html(actualIndex + "/" + actualPageCount);
	},
	updateSelectedPage : function(selIndex) {
		var pageList = $("body").children(".pageList");
		pageList.find("li[type='page'][excluded='false']").css("list-style", "url(\"styles/page.png\")");
		pageList.find("li[pageIndex='" + selIndex + "']")
			.css("list-style", "url(\"styles/page_selected.png\")");
	}
};